/*
 * Copyright (c) 2005, 2014 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/4
 * Revised on 2014/11/15
 */

package agentspace;

import java.io.Serializable;
import java.net.InetAddress;
import java.util.StringTokenizer;

public class AgentIdentifier implements Serializable {
	private static short lastCount = 0;
	private int unique;
	private long time;
	private short count;
	private long addr = 1;

	public AgentIdentifier(int unique, long time, int addr, short count) {
		this.unique = unique;
		this.time = time;
		this.addr = addr;
		this.count = count;
	}

	public AgentIdentifier(Object obj) {
		this();
		this.unique = getHashCode(obj);
	}

	public AgentIdentifier() {
		try {
			if (lastCount == Short.MAX_VALUE) {
				lastCount = Short.MIN_VALUE;
			}
			unique = getUniqueHashCode();
			time = System.currentTimeMillis();
			InetAddress localInetAddress = InetAddress.getLocalHost();
			byte[] ip = localInetAddress.getAddress();
			for (int i = 0; i < ip.length; i++) {
				addr = addr * 0xFF + (((int) ip[i]) & 0xFF);
			}
			addr = addr + 0xFFFFFFFF / 2;
			count = lastCount++;
		} catch (Exception e) {
		}
	}

	public AgentIdentifier(String line) {
		StringTokenizer tokenizer = new StringTokenizer(line, "-");
		if (tokenizer.hasMoreTokens()) {
			this.unique = Integer.parseInt(tokenizer.nextToken());
		}
		if (tokenizer.hasMoreTokens()) {
			this.time = Long.parseLong(tokenizer.nextToken());
		}
		if (tokenizer.hasMoreTokens()) {
			this.addr = Long.parseLong(tokenizer.nextToken());
		}
		if (tokenizer.hasMoreTokens()) {
			this.count = Short.parseShort(tokenizer.nextToken());
		}
	}

	private static int getHashCode(Object obj) {
		return obj.hashCode();
	}

	private static int getUniqueHashCode() {
		return getHashCode(new Object());
	}

	public int hashCode() {
		return (int) time + (int) count;
	}

	public boolean equals(Object obj) {
		if ((obj != null) && (obj instanceof AgentIdentifier)) {
			AgentIdentifier objid = (AgentIdentifier) obj;
			return (unique == objid.unique && count == objid.count
					&& addr == objid.addr && time == objid.time);
		} else {
			return false;
		}
	}

	public String toString() {
		return Integer.toString(unique, 10) + "-" + Long.toString(time, 10)
				+ "-" + Long.toString(addr, 10) + "-"
				+ Integer.toString(count, 10);
	}
}

