package agentspace.discovery.gomi;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import agentspace.discovery.AgentServerInfo;

public class HeartbeatEntryManager {
	final int MAX_THREAD_NUMBER = 3;
	List<HeartbeatEntry> list;
	List<ExpiredListener> listeners;
	ScheduledExecutorService exec;
	ExpiredEntryMonitor monitor;

	public HeartbeatEntryManager() {
		list = new ArrayList<HeartbeatEntry>();
		listeners = new ArrayList<ExpiredListener>();
		exec = Executors.newScheduledThreadPool(MAX_THREAD_NUMBER);
		monitor = new ExpiredEntryMonitor(list, listeners);
        exec.scheduleAtFixedRate(monitor, 0, 100, TimeUnit.MILLISECONDS);
	}
	
	public void stop() {
		monitor.stop();
	}

	public synchronized void addHeartbeatEntry(HeartbeatEntry entry) throws ExpiredEntryAlreadyBoundException {
		if (list == null) {
			list = new ArrayList<HeartbeatEntry>();
		}
		if (list.contains(entry)) {
			throw new ExpiredEntryAlreadyBoundException();
		}
		list.add(entry);
	}
	
	public synchronized void removeHeartbeatEntry(HeartbeatEntry entry) throws NoSuchEntryException {
		if (list == null) {
			throw new NoSuchEntryException();
		}
		list.remove(entry);		
	}
	
	public static void main(String[] args) {
		HeartbeatEntryManager manager = new HeartbeatEntryManager();
		try {
			manager.addHeartbeatEntry(new HeartbeatEntry(null, "hello", 250));
			Thread.sleep(100);
			manager.addHeartbeatEntry(new HeartbeatEntry(null, "good-bye", 310));
		}
		catch (ExpiredEntryAlreadyBoundException ex) {
			ex.printStackTrace();
		} catch (InterruptedException ex) {
			ex.printStackTrace();
		}
	}
	
	public void addExpiredListener(ExpiredListener listener) throws AlreadyBoundException {
		if (listeners == null) {
			listeners = new ArrayList<ExpiredListener>();
		}
		if (listeners.contains(listener)) {
			throw new AlreadyBoundException();
		}
		listeners.add(listener);
	}
	
	public void removeExpiredListener(ExpiredListener listener) throws NoSuchElementException {
		if (listener == null) {
			throw new NoSuchElementException();
		}
		listeners.remove(listener);
	}

}
