package agentspace.discovery;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Date;
import java.util.List;

import agentspace.AgentIdentifier;
import agentspace.multicast.MulticastManager;
import agentspace.multicast.MulticastUDPReceiver;
import agentspace.multicast.MulticastUDPSender;
import agentspace.multicast.Packet;
import agentspace.multicast.SizeLimitExceededPacketException;
import agentspace.registry.AgentRegistry;
import agentspace.registry.AlreadyBoundException;

public class AutonomyManager {
	final protected String groupName = "228.1.2.3";
	final protected int EXPIRE = 3000;
	final protected int CHECKING_INTERVAL = 1000;
	final protected int SENDING_INTERVAL = 2000;
	protected RuntimeRegistry neighbors;
	protected URI uri;
	protected int port;
	protected boolean active = true;
	protected MulticastUDPSender sender;
	protected Thread receiverThread = null;
	protected MulticastManager manager;
	protected MulticastUDPReceiver receiver;
	protected RuntimeDiscoveryExpire expire;
	protected AgentRuntimeHeatbeat heartbeat = null;
	protected AgentRegistry registry;
	protected Packet packet;
	protected List heatbeatList = null;
	
	public AutonomyManager(RuntimeRegistry runtimes, int port) throws IOException {
		this.neighbors = runtimes;
		this.port = port;
		receiver = new MulticastUDPReceiver(groupName, port);
		sender = new MulticastUDPSender(groupName, port);
	}

	public Heartbeat addHeartbeat(Packet packet, int expire, int interval) {
		if (heartbeatList == null) {
			heartbeatList = new ListedLink();
		}
		RuntimeDiscoveryExpire expire = new RuntimeDiscoveryExpire(runtimes, EXPIRE, CHECKING_INTERVAL);
		Heatbeat heartbeatList = new Heatbeat(packet, sender, expire, interval);
		heartbeat = new AgentRuntimeHeatbeat(sender, packet, SENDING_INTERVAL);
		
	

		Packet packet = new Packet();
		packet.setMethod("ADVERTISEMENT");
		packet.setUri(uri);
		packet.setProtocol("AGENTSPACE");
		packet.setVersion("0.3");
		heartbeat = new AgentRuntimeHeatbeat(sender, packet, SENDING_INTERVAL);
	}

	public void search(AgentIdentifier aid) throws IllegalArgumentException, IOException, SizeLimitExceededPacketException, URISyntaxException {
		Packet packet = new Packet();
		packet.setMethod("LOOKUP");
		packet.setUri(new URI("///"+aid.toString()));
		packet.setProtocol("AGENTSPACE");
		packet.setVersion("0.3");
		sender.send(packet.getBytes());
	}
	
	public void send(String protocol, String version, String method, URI uri, String content) throws IllegalArgumentException, IOException, SizeLimitExceededPacketException {
		Packet packet = new Packet();
		packet.setMethod(method);
		packet.setUri(uri);
		packet.setProtocol(protocol);
		packet.setVersion(version);
		sender.send(packet.getBytes());
	}
	
	public void start() {
		active = true;
		expire.start();
		if (receiverThread == null) {
			receiverThread = new Thread(executeReceiving);
			receiverThread.start();
		}
		heartbeat.start();
	}

	public void stop() {
		active = false;
		heartbeat.stop();
		receiverThread.interrupt();
		receiverThread = null;
		expire.stop();
	}

	Runnable executeReceiving = new Runnable() {
		public void run() {
			while (active) {
				try {
					byte[] data = receiver.receive();
					Packet packet;
					packet = new Packet(data);
//					System.out.println("AgentRuntimeDiscovery: receiving at "+new Date() + ":(" + packet+")");
					if (packet.getProtocol().equalsIgnoreCase("AGENTSPACE")) {
						if (packet.getMethod().equalsIgnoreCase("ADVERTISEMENT")) {
							URI uri = packet.getUri();
							AgentServerInfo asi = (AgentServerInfo)neighbors.lookup(uri); 
							if (asi == null) {
								asi = new AgentServerInfo(uri);
								try {
									neighbors.add(uri, asi);
//									joined(uri, asi);
								} 
								catch (AlreadyBoundException e) {
									e.printStackTrace();
								}
							}
							else {
								asi.resetLastTime();
							}
						}
						else if (packet.getMethod().equalsIgnoreCase("LOOKUP")) {
							
						}
					}
					else if (packet.getProtocol().equalsIgnoreCase("MOBILESPACES")) {
						
					}
				} 
				catch (SizeLimitExceededPacketException e) {
					e.printStackTrace();
				}
				catch (URISyntaxException e) {
					e.printStackTrace();
				}
				catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	};
}