package agentspace.discovery.scrap;

import java.net.URI;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import agentspace.packet.Packet;
import agentspace.registry.AlreadyBoundException;
import agentspace.registry.NotBoundException;

public class HeartbeatExpire implements PacketListener, Runnable {
	protected boolean active = true;
	protected String protocol;
	protected String method;
	protected int expired = -1;
	protected int interval = -1;
	protected HeartbeatExpireManager manager;
	protected long lastTime = -1;
	protected long expireTime = -1;
	protected List<ExpiredListener> listeners = null;
	
	public HeartbeatExpire(HeartbeatExpireManager manager, String protocol, String method, int expired, int interval) {
		this.manager = manager;
		this.protocol = protocol;
		this.method = method;
		this.expired = expired;
		this.interval = interval;
		listeners = new LinkedList<ExpiredListener>();
	}
	
	public void addExpiredListener(ExpiredListener listener) {
		listeners.add(listener);
	}

	public void removeExpiredListener(ExpiredListener listener) {
		listeners.remove(listener);
	}

	public void start() throws AlreadyBoundException {
		manager.addPacketListener(this, protocol, method);
		
	}

	public synchronized void received(Packet packet) {
		lastTime = System.currentTimeMillis();
	}
	
	public boolean isExpired(long time) {
		if (time - lastTime > expireTime) {
			return true;
		}
		else {
			return false;
		}
	}
	
	public void run() {
		while(active) {
			try {
				if (System.currentTimeMillis() - lastTime > expireTime) {
					for (ExpiredListener i : listeners) {

					}
				}
				
				
				Iterator<URI> i = registry.getKeys().iterator();
				while (i.hasNext()) {
					URI uri = (URI)i.next();
					AgentServerInfo asi = registry.lookup(uri);
					if (asi != null) {
//						System.out.println("AgentRuntimeDiscovery: System.currentTimeMillis:"+System.currentTimeMillis()+" - asi.getLastTime:"+asi.getLastTime() +" = "+ (System.currentTimeMillis() - asi.getLastTime()) +" >? "+expired);
						if (System.currentTimeMillis() - asi.getLastTime() > expired) {
							registry.remove(uri);
						}
					}
				}
				try {
					Thread.sleep(interval);
				} 
				catch (InterruptedException e) {
					break;
				}
			} 
			catch (NotBoundException e) {
				e.printStackTrace();
			}

		}
	}
}
