package agentspace.discovery.scrap;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import agentspace.AgentIdentifier;
import agentspace.discovery.AgentServerInfo;
import agentspace.discovery.RuntimeDiscoveryExpire;
import agentspace.discovery.RuntimeRegistry;
import agentspace.multicast.MulticastManager;
import agentspace.packet.Packet;
import agentspace.packet.PacketListener;
import agentspace.packet.SizeLimitExceededPacketException;
import agentspace.packet.multicast.MulticastUDPReceiver;
import agentspace.packet.multicast.MulticastUDPSender;
import agentspace.registry.AgentRegistry;
import agentspace.registry.AlreadyBoundException;
import agentspace.registry.NotBoundException;

public class HeartbeatExpireManager implements PacketListener, Runnable {
	final protected String defaultGroupName = "228.1.2.3";
	final protected int EXPIRE = 3000;
	final protected int CHECKING_INTERVAL = 1000;
	final protected int SENDING_INTERVAL = 2000;
	final protected String PROTOCOL = "AGENTSPACE";
	final protected String VERSION = "0.3";
	final protected String ADVERTISE_METHOD = "ADVERTISE";
	final protected String LOOKUP_METHOD = "LOOKUP";
		
	protected MulticastPacketManager manager;
	protected int port;
	protected RuntimeRegistry neighbors;
	protected boolean active = true;
	protected String group;

	protected List<Heartbeat> beats;
	
	public HeartbeatExpireManager(Packet packet, String group, int port, int interval) throws IOException {
		Heartbeat heartbeat = new Heartbeat(packet, interval);
		manager = new MulticastPacketManager(port, group); 
		this.port = port;
	}
	
	class HeartbeatPacketLisenerImpl implements PacketListener {
		public void received(Packet packet) {
			// TODO Auto-generated method stub
			
		}
	}
	
	
	PacketListener listener = new PacketListener(void received() {});
	
	public void start() {
		manager.addPacketListener(null, ADVERTISE_METHOD, ADVERTISE_METHOD)

	}	

	public void run() {
		while (active) {
		}
	}

	public void received(Packet packet) {
		// TODO Auto-generated method stub
		
	}

	
	public Heartbeat startHeartbeat(Packet packet, int interval) {
		Heartbeat heartbeat = new Heartbeat(manager, interval);
		heartbeat.setPacket(packet);
		heartbeat.start();
		if (beats == null) {
			beats = new LinkedList<Heartbeat>();
		}
		beats.add(heartbeat);
		return heartbeat;
	}
	
	public void stopHeartbeat(Heartbeat beat) {
		beat.stop();
		beats.remove(beat);
	}
	
	public Heartbeat addHeartbeatListener(HeartbeatListener listener, String protocol, String method, int expire) {
		expire = new RuntimeDiscoveryExpire(neighbors, EXPIRE, CHECKING_INTERVAL);
		manager = new MulticastPacketManager(port, defaultGroupName);
	}
		
	public void removeHeartbeatListener(Heartbeat listener, String protocol, String method) {}
	
	
	public HeartbeatExpireManager(MulticastPacketManager manager, String protocol, String method, int expire) {
		this.manager = manager;
	}
	
	


	public List<AgentServerInfo> lookup(AgentIdentifier aid) throws IllegalArgumentException, IOException, SizeLimitExceededPacketException, URISyntaxException {
		PacketListener listener = new PacketListener() {
			public void received(Packet packet) {
							
			}
			public AgentServerInfo getLocation(int timeout) {} 
		};
		manager.addPacketListener(listener);
		Packet packet = new Packet();
		packet.setMethod(LOOKUP_METHOD);
		packet.setUri(new URI("///"+aid.toString()));
		packet.setProtocol(PROTOCOL);
		packet.setVersion(VERSION);
		manager.multicast(packet);
		return getLocation();
	}
	
	public void send(String protocol, String version, String method, URI uri, String content) throws IllegalArgumentException, IOException, SizeLimitExceededPacketException {
		Packet packet = new Packet();
		packet.setMethod(method);
		packet.setUri(uri);
		packet.setProtocol(protocol);
		packet.setVersion(version);
		sender.send(packet.getBytes());
	}
	
	public void start() {
		active = true;
		expire.start();
		if (receiverThread == null) {
			receiverThread = new Thread(executeReceiving);
			receiverThread.start();
		}
		heartbeat.start();
	}

	public void stop() {
		active = false;
		heartbeat.stop();
		receiverThread.interrupt();
		receiverThread = null;
		expire.stop();
	}
	



	class PacketListenerEntry {
		PacketListener packetLisener;
		String protocol;
		String method;
		public PacketListenerEntry(PacketListener packetLisener, String protocol, String method) {
			this.packetLisener = packetLisener;
			this.protocol = protocol;
			this.method = method;
		}
		public PacketListener getPacketLisener() {
			return packetLisener;
		}
		public String getProtocol() {
			return protocol;
		}
		public String getMethod() {
			return method;
		}
	}


}