package agentspace.discovery;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.net.URI;

import agentspace.registry.AlreadyBoundException;
import agentspace.registry.NotBoundException;

public class RuntimeRegistry {
	protected Map<URI, AgentServerInfo> neighbours = null;
			
	public RuntimeRegistry() {
		neighbours = new LinkedHashMap<URI, AgentServerInfo>();
	}

	public void add(URI uri, AgentServerInfo asi) throws AlreadyBoundException {
		System.out.println("RuntimeRegistry: add("+asi+")");
		if (neighbours.containsKey(uri) || (asi != null && neighbours.containsValue(asi))) {
			throw new AlreadyBoundException(uri.toString());
		}
		synchronized(neighbours) {
			neighbours.put(uri, asi);
		}
		if (listeners != null) {
			Iterator<RuntimeRegistryListener> i = listeners.iterator();
			while(i.hasNext()) {
				RuntimeRegistryEvent evt = new RuntimeRegistryEvent(uri, asi);
				i.next().added(evt);
			}
		}
	}
	
	public void remove(URI uri) throws NotBoundException {
		System.out.println("RuntimeRegistry: remove("+uri+")");
		AgentServerInfo asi = neighbours.get(uri);
		if (asi == null) {
			throw new NotBoundException(uri.toString());
		}
		synchronized(neighbours) {
			neighbours.remove(uri);
		}
		if (listeners != null) {
			Iterator<RuntimeRegistryListener> i = listeners.iterator();
			while(i.hasNext()) {
				RuntimeRegistryEvent evt = new RuntimeRegistryEvent(uri, asi);
				i.next().removed(evt);
			}
		}
	}

	public AgentServerInfo[] getNeighbours() {
		return (AgentServerInfo[])neighbours.values().toArray();
	}

	public AgentServerInfo lookup(URI uri) {
		return neighbours.get(uri);
	}

	public Set<URI> getKeys() {
		return neighbours.keySet();
	}

	public Collection<AgentServerInfo> values() {
		return neighbours.values();
	}

	public Iterator<AgentServerInfo> iterator() {
		return neighbours.values().iterator();
	}

	List<RuntimeRegistryListener> listeners = null;

	public void addRuntimeRegistryListener(RuntimeRegistryListener l) {
		if (listeners == null) {
			listeners = new ArrayList<RuntimeRegistryListener>();
		}
		listeners.add(l);
	}

	public void removeRuntimeRegistryListener(RuntimeRegistryListener l) {
		if (listeners != null) {
			listeners.remove(l);
		}
		if (listeners.size() == 0) {
			listeners = null;
		}
	}
}
