/*
 * Copyright (C) 2013 Ichiro Satoh (ichiro@nii.ac.jp)
 * All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting
 * documentation.
 *
 * FREE USE OF THIS SOFTWARE IS ALLOWED IN ITS "AS IS" CONDITION.
 * BUT WITHOUT WARRANTY. THE AUTHORS DISCLAIM ANY LIABILITY OF ANY 
 * KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS 
 * SOFTWARE.
 */

package agentspace.dmi;

import java.io.IOException;
import java.io.OptionalDataException;
import java.io.StreamCorruptedException;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

import agentspace.Message;
import agentspace.runtime.loader.HashtableClassLoader;
import agentspace.runtime.loader.MarshallingUtil;


public class Proxy {
	Object obj = null;

	public Proxy(Message msg) throws InstantiationException, ClassNotFoundException, SecurityException, NoSuchMethodException, IllegalArgumentException, IllegalAccessException, InvocationTargetException {
		Class<?> cl = Class.forName(msg.getName());
		Constructor<?> cst = cl.getConstructor(msg.getTypes());
		obj = cst.newInstance(msg.getArgs());
	}

	public Object invoke(Message msg) throws NoSuchObjectException, NoSuchMethodException, InvocationTargetException, IllegalArgumentException, IllegalAccessException { 
		if (obj == null) {
			throw new NoSuchObjectException(msg.getName());
		}
		Method meth = getMethod(obj.getClass(), msg.getName(), msg.getTypes(), false);
		if (meth == null) {
			throw new NoSuchMethodException("No Such Method: "+msg.getName());
		}
		return meth.invoke(obj, msg.getArgs());
	}

	protected static Method getMethod(Class<? extends Object> targetClass, String methodName, Class<?>[] classes, boolean asStatic) throws NoSuchMethodException {
		int length = 0;
		if (classes != null) {
			length = classes.length;
		}
		Method[] meths = targetClass.getMethods();
		for( int i = 0 ; i < meths.length ; i++ ) {
			if (meths[i].getName().equals(methodName) && meths[i].getParameterTypes().length == length && (!asStatic || (meths[i].getModifiers() & Modifier.STATIC) != 0)) {
				Class<?>[] methodArgClasses = meths[i].getParameterTypes();
				boolean isMatch = true;
				for(int j = 0 ; isMatch && j < length ; j++ ) {
					isMatch = isMatch && (classes[j] == null ||	methodArgClasses[j].equals(classes[j]) || methodArgClasses[j].isAssignableFrom(classes[j]));
				}
				if (isMatch) {
					return meths[i];
				}
			}
		}
		throw new NoSuchMethodException(methodName + " in " + targetClass);
	}

	public static byte[] toByte(Message msg) throws IOException {
		return MarshallingUtil.marshal(msg); 
	}

	public static Message toMessage(byte[] data, HashtableClassLoader loader) throws ClassNotFoundException, StreamCorruptedException, OptionalDataException, IOException {
		return (Message)MarshallingUtil.unmarshal(data, loader);
	}

	public String toString() {
		return obj.toString();
	}
}