/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/12
 */


package agentspace.monitor.dialog;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;

import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;

import agentspace.discovery.AgentServerInfo;
import agentspace.discovery.RuntimeRegistry;

public class AgentDestinationDialog implements ActionListener {
	JTextField textField;
	JComboBox combo;
	public URL url = null;

	public AgentDestinationDialog(RuntimeRegistry registry, JFrame frame, URL url) {
		textField = new JTextField(30);
		textField.setText(url.toString());
		textField.setBorder(new TitledBorder("go to URL:"));
		
		int size = registry.values().size();
		String[] destinations = new String[size+1];
		destinations[0] = "no selected";
		Iterator<AgentServerInfo> itr = registry.values().iterator();
		for (int i = 1 ; itr.hasNext() ; i++) {
			AgentServerInfo asi = itr.next();
			destinations[i] = "matp://" + asi.getHostName() + ":" + asi.getPort();
			System.out.println("AgentDestinationDialog: destinations["+i+"]="+destinations[i]);
		}
		combo = new JComboBox(destinations);
		combo.setPreferredSize(new Dimension(80, 30));
		combo.addActionListener(this);
	
		Object[] obj = {textField, combo};
		int ans = JOptionPane.showConfirmDialog(frame.getContentPane(), obj, "Dialog", JOptionPane.OK_CANCEL_OPTION);
		if (ans == JOptionPane.YES_OPTION) {
			try {
				url = new URL(textField.getText());
//				System.out.println("AgentDestinationDialog: url = "+url);
			} 
			catch (MalformedURLException e) {
				e.printStackTrace();
				url = null;
			}
		} 
		else if (ans == JOptionPane.CANCEL_OPTION) {
			url = null;
		}
		this.url = url;
	}

	public URL getURL() {
		return url;
	}
	
	public void actionPerformed(ActionEvent evt) {
		String selectedDestination;
		if (combo.getSelectedIndex() == -1){
			selectedDestination = textField.getText();
		}else{
			selectedDestination = (String)combo.getSelectedItem();
			textField.setText(selectedDestination);
		}
		
	}
}

