/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/13
 */

package agentspace.monitor.network;

import java.awt.BorderLayout;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import agentspace.discovery.AgentServerInfo;
import agentspace.discovery.RuntimeRegistryEvent;
import agentspace.discovery.RuntimeRegistryListener;
//import agentspace.runtime.AgentContainer;

public class AgentNetworkView extends JPanel implements FocusListener, MouseListener, KeyListener, RuntimeRegistryListener {
	JTable networkTable;
	AgentTableNetworkModel tableModel = null;

	public AgentNetworkView(URI uri) {
		setLayout(new BorderLayout());
		tableModel = new AgentTableNetworkModel();
		networkTable = new JTable(tableModel);
		
		tableModel.setTable(networkTable);
		networkTable.setSize(600, 150);
		networkTable.addMouseListener(this);
		networkTable.addKeyListener(this);

		JScrollPane scrollpane = new JScrollPane(networkTable);
		add(scrollpane);
		setVisible(true);
		
		JFrame frame = new JFrame();
		frame.setTitle("Testing mode: Heartbeat monitor: "+uri.toString());
		frame.getContentPane().add(this);
		frame.setSize(600, 200);
		frame.setVisible(false);

	}

	public int count() {
		return tableModel.getRowCount();
	}

	synchronized public void addElement(AgentServerInfo asi) {
		tableModel.addElement(asi);
	}

	synchronized public void removeElement(AgentServerInfo asi) {
		tableModel.removeElement(asi);
	}

	synchronized public void changeElement(AgentServerInfo asi) {
		tableModel.changeElement(asi);
	}

	synchronized public void removeElementAt(int row) {
		tableModel.removeElementAt(row);
	}

	public void mouseClicked(MouseEvent evt) {
		JTable table = (JTable)evt.getComponent();
		System.out.println("AgentNetworkView: MouseEvent mouseClicked("+evt+")");
	}
	public void mousePressed(MouseEvent evt) {}
	public void mouseReleased(MouseEvent evt) {}
	public void mouseEntered(MouseEvent evt) {}
	public void mouseExited(MouseEvent evt) {}
	public void focusGained(FocusEvent evt) {}
	public void focusLost(FocusEvent evt) {}
	public void keyTyped(KeyEvent evt) {}
	public void keyPressed(KeyEvent evt) {}
	public void keyReleased(KeyEvent evt) {}

	public void added(RuntimeRegistryEvent evt) {
//		System.out.println("AgentNetworkView: added("+evt+")........ADD......");
		addElement(evt.getAgentServerInfo());
	}

	public void removed(RuntimeRegistryEvent evt) {
//		System.out.println("AgentNetworkView: removed("+evt+")........REMOVE.......");
		removeElement(evt.getAgentServerInfo());
	}

	public static void main(String[] args) {
		try {
			AgentNetworkView view = new AgentNetworkView(new URI("matp://localhost:1"));
			List<AgentServerInfo> list = new ArrayList<AgentServerInfo>();
			for (int i = 0 ; i < 5 ; i++ ) {
				try {Thread.sleep(500);}catch (Exception e) {}
				AgentServerInfo asi = new AgentServerInfo(new URI("matp://localhost:"+(7000+i)));
				list.add(asi);
				view.addElement(asi);
			}
			Iterator<AgentServerInfo> itr = list.iterator();
			while (itr.hasNext()) {
				try {Thread.sleep(500);}catch (Exception e) {}
				AgentServerInfo asi = itr.next();
//				System.out.println("AgentNetwork View: asi="+asi);
				view.removeElement(asi);
			}
			list.clear();
		} 
		catch (URISyntaxException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
    }

}