/*
 * Copyright (c) 2005, 2010 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

package agentspace.network.protocol;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;

public class RequestParser extends Request {
    protected Map<String, String> attributes = null;

    public RequestParser(InputStream in) throws IOException {
        int contentLength = 0;
        String statusLine = Util.readLine(in);
//        System.out.println("RequestParser: statusLine = "+statusLine);
        StringTokenizer tokens = new StringTokenizer(statusLine, " ");
        if (tokens.countTokens() < 2) {
//          System.err.println("RequestParser: invalid status line = "+statusLine);
            throw new IOException("Invalid Request: " + statusLine);
        }
        requestMethod = tokens.nextToken();
        requestURI = tokens.nextToken();
        StringTokenizer st = new StringTokenizer(tokens.nextToken().trim(), "/");
        requestProtocol = st.nextToken().trim().toUpperCase();
        requestVersion = st.nextToken().trim();
        while (true) {
            if (attributes == null) {
                attributes = new HashMap<String, String>();
            }
            String attributeLine = Util.readLine(in);
            if (attributeLine == null || attributeLine.length() == 0) {
                break;
            }
            int len = attributeLine.length();
            int pos = attributeLine.indexOf(":");
            if (pos != -1) {
                String name = attributeLine.substring(0, pos).trim();
                String property = attributeLine.substring(pos + 1, len).trim();
                attributes.put(name, property);
                if (name.equalsIgnoreCase("Content-Length")) {
                    try {
                        contentLength = new Integer(property).intValue();
                    }
                    catch (NumberFormatException ex) {}
                }
            }
        }
        if (contentLength > 0) {
            java.io.ByteArrayOutputStream baos = new java.io.ByteArrayOutputStream();
            for (int i = 0 ; i < contentLength ; i++ ) {
                int result = in.read();
                if (result < 0) {
                    throw new IOException("Invalid Content");
                }
                baos.write((byte)result);
            }
            baos.flush();
            requestContent = baos.toByteArray();
        }
    }
}
