/*
 * Copyright (c) 2005, 2010 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

package agentspace.network.protocol;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

public class Response {
    protected String responseProtocol = null;
    protected String responseVersion = null;
    protected int responseCode = -1;
    protected String responseComment = null;
    protected byte[] responseContent = null;
    protected Map<String, String> attributes = null;

    public Response() {}

    public String getVersion() {
      return responseVersion;
    }

    public String getProtocol() {
      return responseProtocol;
    }
    
    public int getCode() {
        return responseCode;
    }

    public String getComment() {
        return responseComment;
    }

    public Map<String, String> getAttributes() {
        return attributes;
    }
    
    public String getAttribute(String name) {
        if (attributes == null) {
            return null;
        }
        return attributes.get(name.toUpperCase());
    }

    public void setAttribute(String name, String value) {
        if (attributes == null) {
        	attributes = new HashMap<String, String>();
        }
        attributes.put(name.toUpperCase(), value);
    }

    public int length () {
        if (responseContent != null) {
            return responseContent.length;
        }
        else {
            return -1;
        }
    }
    
    public byte[] getContent() {
        return responseContent;
    }
    
    public String toString() {
      	return "protocol:"+responseProtocol+", code:"+responseCode+", comment:"+responseComment;
      }

    public byte[] toByteArray() {
        byte[] data = null;
    	try {
    		ByteArrayOutputStream baos = new ByteArrayOutputStream(); 
    		DataOutputStream dos = new DataOutputStream(baos);
    		
    		String headerLine = "";
    		if (responseProtocol != null && responseVersion != null) {  
    			headerLine = headerLine + responseProtocol + "/" + responseVersion + " ";
    		}
    		if (responseCode > 0) {  
    			headerLine = headerLine + responseCode;
    		}
    		if (responseComment != null) {
    			headerLine = headerLine + " " + responseComment + "\r\n";
    		}
    		else {
    			headerLine = headerLine + "\r\n";
    		}
    		dos.writeBytes(headerLine);
    		
    		if (responseContent != null) {
    			String value = getAttribute("Content-Length");
    			if (value == null) {
    				attributes.put("Content-Length", String.valueOf(responseContent.length));
    			}
    		}

    		String attributeLine = "";
    		if (attributes != null && !attributes.isEmpty()) {
    			Iterator<String> i = attributes.keySet().iterator();
    			while(i.hasNext()) {
    				String attributeName = i.next();
    				String attributeValue = attributes.get(attributeName); 
    				attributeLine = attributeName + ": " +  attributeValue + "\r\n";
    			}
				dos.writeBytes(attributeLine);
    		}
    		if (headerLine.length() != 0) {
    			dos.writeBytes("\r\n");		    	
    		}
    		if (responseContent != null) {
    			dos.write(responseContent, 0, responseContent.length);
    		}
    		dos.close();
    		data = baos.toByteArray();
    	}
    	catch (IOException ex) {
    		ex.printStackTrace();
    	}
    	return data;
    }
}
