package agentspace.packet;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import agentspace.registry.AlreadyBoundException;
import agentspace.registry.NotBoundException;

public class PacketExpireManager implements Runnable {
	protected boolean active = true; 
	protected int interval = -1;
	protected Thread expireThread = null;
	protected List<ExpireListener> listeners;
	protected List<PacketExpireEntry> entries;

	public PacketExpireManager(int interval) {
		this.entries = entries;
		this.interval = interval;
		listeners = new LinkedList<ExpireListener>();
		entries = new LinkedList<PacketExpireEntry>(); 
	}

	public void start() {
		active = true;
		if (expireThread == null) {
			expireThread = new Thread(this);
			expireThread.start();
		}
	}

	public void stop() {
		active = false;
		if (expireThread != null) {
			expireThread.interrupt();
			expireThread = null;
		}
	}
	
	public synchronized void addExpireEntry(PacketExpireEntry entry) {
		entries.add(entry);
	}

	public synchronized void removeExpireEntry(PacketExpireEntry entry) {
		entries.remove(entry);
	}

	public void addExpireListener(ExpireListener l) throws AlreadyBoundException {
		if (listeners.contains(l)) {
			throw new AlreadyBoundException(l.toString());
		}
		listeners.add(l);
	}

	public void removeExpireListener(ExpireListener l) throws NotBoundException {
		if (!listeners.contains(l)) {
			throw new NotBoundException(l.toString());
		}
		listeners.remove(l);
	}

	public void run() {
		while(active) {
			Iterator<PacketExpireEntry> i = entries.iterator();
			while (i.hasNext()) {
				PacketExpireEntry entry = i.next();
				if (System.currentTimeMillis() - entry.getLastTime()  > entry.getLastTime()) {
					Iterator<ExpireListener> it = listeners.iterator();
					while (it.hasNext()) {
						ExpireListener listener = it.next();
						listener.expired(new PacketExpiredEvent(entry));
						entries.remove(entry);
					}
				}
			}
			try {
				Thread.sleep(interval);
			} 
			catch (InterruptedException e) {
				break;
			}
		}
	}
}
