/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/5
 */

package agentspace.runtime;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import agentspace.Future;
import agentspace.Message;
import agentspace.runtime.loader.MethodUtil;

public class AgentMessageHandler implements Runnable {
	protected Message msg = null;
	private Object target = null;
	private Object result = null;
	protected boolean finished = false;
	
	public AgentMessageHandler(Message msg) {
		this(msg.getTarget(), msg);
	}
	
	public AgentMessageHandler(Object target, Message msg) {
		this.msg = msg;
		this.target = target;
		if (msg.getFuture() == null) {
			msg.setFuture(new Future());
		}
	}

	public synchronized void run() {
		try {
			Object[] argObjects = msg.getArgs();
			Class<?>[] argClasses = msg.getTypes();
			Method method = MethodUtil.getMethod(target.getClass(), msg.getName(), argClasses, false);
			result = MethodUtil.invoke(target, method, argObjects);
			msg.getFuture().setReply(result);
		}
		catch (NoSuchMethodException e) {
			msg.setException(e);
		}
		catch (InvocationTargetException e) {
			msg.setException(e);
		}
		catch (IllegalAccessException e) {
			msg.setException(e);
	  	}
		catch (IllegalArgumentException e) {
			msg.setException(e);
		}
	}
	
	public void setException(Exception except) {
		msg.setException(except);
	}
	
	public Exception getException() {
		return msg.getException();
	}
	
	public Message getMessage() {
		return msg;
	}

	public void setFuture(Future future) {
		msg.setFuture(future);
	}
	
	public Future getFuture() { 
		return msg.getFuture();
	}
	
	public void setResult(Object result) {
		msg.getFuture().setReply(result);
	}
	
	public Object getResult() {
		return msg.getFuture().getReply();
	}
}
