/*
 * Copyright (c) 2005, 2015, 2020 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/5
 */

package agentspace.runtime;

public class AgentStatus {
	  final public static int UNKNOWN      = 0x00;

	  /* basic status */
	  final public static int LIFE_MASK     = 0xFF;
	  final public static int UNINITIALIZED = 0x01;
	  final public static int CREATING      = 0x02;
	  final public static int NORMAL        = 0x03;	  	  
	  final public static int SERIALIZING   = 0x04;
	  final public static int SERIALIZED    = 0x05;
	  final public static int UNSERIALIZING = 0x06;
	  final public static int UNSERIALIZED  = 0x07;
	  final public static int TERMINATING   = 0x08;	  
	  final public static int TERMINATED    = 0x09;
	  
	  /* mobility status */
	  final public static int LEAVING       = 0x0A;
	  final public static int LEFT          = 0x0B;
	  final public static int MOVING        = 0x0C;
	  final public static int ARRIVING      = 0x0D;
	  final public static int ARRIVED       = 0x0E;
	  
	  /* duplication status */
	  final public static int DUPLICATING   = 0x0F;
	  final public static int DUPLICATED    = 0x10;
	  final public static int PARENT        = 0x11;
	  final public static int CHILD         = 0x12;

	  /* hierarchy status */
	  final public static int ADDING        = 0x13;
	  final public static int ADDED         = 0x14;
	  final public static int REMOVING      = 0x15;
	  final public static int REMOVED       = 0x16;
	  
	  final public static int HALT          = 0x41;
	  final public static int EXCEPTION     = 0x42;

	  final public static int PROCESS_MASK = 0xF << 8;
	  final public static int READY        = 0x0 << 8;
	  final public static int BUSY         = 0x1 << 8;
	  final public static int EXCLUSIVE    = 0x2 << 8;

	  public static String getLifeName(int status) {
	  	switch(status & LIFE_MASK) {
	  	case UNINITIALIZED:
	  		return "uninitialized";
	  	case CREATING:
	  		return "creating";
	  	case NORMAL:
	  		return "normal";
	  	case SERIALIZING:
	  		return "serializing";
	  	case SERIALIZED:
	  		return "serialized";
	  	case UNSERIALIZING:
	  		return "unserializing";
	  	case UNSERIALIZED:
	  		return "unserialized";
	  	case TERMINATING:
	  		return "terminating";
	  	case TERMINATED:
	  		return "terminated";
	  	case LEAVING:
	  		return "leaving";
	  	case LEFT:
	  		return "left";
	  	case ARRIVING:
	  		return "arriving";
	  	case ARRIVED:
	  		return "arriving";
	  	case MOVING:
	  		return "moving";
	  	case DUPLICATING:
	  		return "duplicating";
	  	case DUPLICATED:
	  		return "duplicated";
	  	case PARENT:
	  		return "parent";
	  	case CHILD:
	  		return "child";
	  	case ADDING:
	  		return "adding";
	  	case ADDED:
	  		return "added";
	  	case REMOVING:
	  		return "removing";
	  	case REMOVED:
	  		return "removed";
	  	case EXCEPTION:
	  		return "exceptional";
	  	case HALT:
	  		return "halt";
	  	default:
	  		return "unknown";
	  	}
	  }
	  	  
	  public static String getProcessName(int status) {
	    switch(status & PROCESS_MASK) {
	    case READY:
	      return "ready";
	    case BUSY:
	      return "busy";
	    case EXCLUSIVE:
	      return "exclusive";
	    default:
	      return "unknown";
	    }
	  }

	  public static String getName(int id) {
	    String result = "";
	    String life = getLifeName(id);
	    if (life != null) {
	      result = result + "[" + life + "]";
	    }
	    String process = getProcessName(id);
	    if (process != null) {
	      result = result + "[" + process + "]";
	    }
	    return result;
	  }

	  static public int getLifeStatus(String name) {
	  	int result = UNINITIALIZED; 
	  	if (name.equalsIgnoreCase("uninitialized")) {
	  		result = UNINITIALIZED; 
	  	}
	  	else if (name.equalsIgnoreCase("creating")) {
	  		result = CREATING;
	  	}
	  	else if (name.equalsIgnoreCase("normal")) {
	  		result = NORMAL;
	  	}
	  	else if (name.equalsIgnoreCase("serializing")) {
	  		result = SERIALIZING;
	  	}
	  	else if (name.equalsIgnoreCase("serialized")) {
	  		result = SERIALIZED;
	  	}
	  	else if (name.equalsIgnoreCase("leaving")) {
	  		result = LEAVING;
	  	}
	  	else if (name.equalsIgnoreCase("moving")) {
	  		result = MOVING;
	  	}
	  	else if (name.equalsIgnoreCase("arriving")) {
	  		result = ARRIVING;
	  	}
	  	else if (name.equalsIgnoreCase("terminating")) {
	  		result = TERMINATING;
	  	}
	  	else if (name.equalsIgnoreCase("terminated")) {
	  		result = TERMINATED;
	  	}
	  	else if (name.equalsIgnoreCase("duplicating")) {
	  		result = DUPLICATING;
	  	}
	  	else if (name.equalsIgnoreCase("duplicated")) {
	  		result = DUPLICATED;
	  	}
	  	else if (name.equalsIgnoreCase("parent")) {
	  		result = PARENT;
	  	}
	  	else if (name.equalsIgnoreCase("child")) {
	  		result = CHILD;
	  	}
	  	else if (name.equalsIgnoreCase("removing")) {
	  		result = REMOVING;
	  	}
	  	else if (name.equalsIgnoreCase("removed")) {
	  		result = REMOVED;
	  	}
	  	else if (name.equalsIgnoreCase("adding")) {
	  		result = ADDING;
	  	}
	  	else if (name.equalsIgnoreCase("added")) {
	  		result = ADDED;
	  	}
	  	else if (name.equalsIgnoreCase("exceptional")) {
	  		result = EXCEPTION;
	  	}
	  	else if (name.equalsIgnoreCase("halt")) {
	  		result = HALT;
	  	}
	  	return result;
	  }	  
	  
}
