package agentspace.shell;

import java.util.StringTokenizer;
import java.util.Vector;

import agentspace.context.ContextManager;
import agentspace.shell.event.ShellEvent;
import agentspace.shell.event.ShellListener;

public class AgentShell extends Shell implements Runnable {
	protected ContextManager context;

	public AgentShell(ContextManager context) {
		this(context, null);
	}

	public AgentShell(ContextManager context, String command) {
		super();
		this.context = context;
		if (command != null) {
			evaluate(command);
		}
	}

	public void evaluate(String line) {
		if (line != null) {
			line = line.trim();
			CommandHandler cmd = null;
			StringTokenizer tokenizer = new StringTokenizer(line, ";&", true);
			while(tokenizer.hasMoreTokens()) {
				String command = tokenizer.nextToken();
				cmd = new AgentCommandHandler(context, command);
				if (tokenizer.hasMoreTokens() && tokenizer.nextToken().endsWith("&")) {
					//	System.out.println("AgentShell[&]: "+command);
					commandInputed(command);
					(new Thread(cmd)).start();
				}
				else {
					//	System.out.println("AgentShell[;]: "+command+" context = "+context);
					commandInputed(command);
					cmd.run();
					try {
						resultOutputed(cmd.getResult());
					}
					catch (Exception e) {
						resultOutputed(e.getMessage());
					}
				}
			}
		}
	}

	protected transient Vector<ShellListener> listeners = new Vector<ShellListener>();
	public synchronized void addShellListener(ShellListener listener) {
		listeners.addElement(listener);
	}

	public synchronized void removeShellListener(ShellListener listener) {
		listeners.removeElement(listener);
	}

	public void commandInputed(String command) {
		ShellEvent evt = new ShellEvent(this, command);
		Vector<ShellListener> clonedListeners = (Vector<ShellListener>)listeners.clone();
		for( int i = 0 ; i < clonedListeners.size() ; i++ ) {
			((ShellListener)clonedListeners.elementAt(i)).commandInputed(evt);
		}
	}

	public void resultOutputed(String result) {
		ShellEvent evt = new ShellEvent(this, result);
		Vector<ShellListener> clonedListeners = (Vector<ShellListener>)listeners.clone();
		for( int i = 0 ; i < clonedListeners.size() ; i++ ) {
			((ShellListener)clonedListeners.elementAt(i)).resultOutputed(evt);
		}
	}
}

