package agentspace.timer;

public class Timer {
    private long expiration;
    private volatile boolean lockForRemoval;
    private TimerListener listener;

    public Timer(TimerListener listener, long duration) {
    	this.listener = listener;
        expiration = System.currentTimeMillis() + duration;
        lockForRemoval = false;
    }
    
    public boolean renew(long duration) {
        synchronized (this) {
            if (lockForRemoval) {
                return false;
            }
            long newExpiration = System.currentTimeMillis() + duration;
            if (newExpiration > expiration) {
                expiration = newExpiration;
            }
            return true;
        }
    }
    
    public TimerListener getTimerListener() {
    	return listener;
    }
    
    public boolean isExpired() {
    	return isExpired(System.currentTimeMillis());	
    }
    
    public boolean isExpired(long now) {
        synchronized (this) {
        	if (expiration < now) {
        		lockForRemoval = true;
    			return true;
    		}
    		else {
	    		return false;
	   		}
        }
    }
}
