package agentspace.timer;

import java.util.Enumeration;
import java.util.Hashtable;

public class TimerManager implements Runnable {
	private Hashtable<TimerListener, Timer> notifyTable = null;
	protected long period = 1000;
	protected Thread timerThread = null;
	protected Timer timer = null;
	protected boolean active = false;
	
	public TimerManager() {
		notifyTable = new Hashtable<TimerListener, Timer>();
	}

	public TimerManager(long period) {
		this();
		this.period = period;
	}

	public void startTimerListener(TimerListener listener, long duration) {
		Timer timer = notifyTable.get(listener);
		if (timer == null) {
			timer = new Timer(listener, duration);
			notifyTable.put(listener, timer);
		}
		if (!active) {
			active = true;
			if (timerThread == null) {
				timerThread = new Thread(this);
			}
			if (!timerThread.isAlive()) {
				timerThread.start();
			}
		}
		if (timer.isExpired()) {
			expired(timer);
		}
	}
	
	public void stopTimerListener(TimerListener listener) throws NoSuchTimerException {
		if (listener == null) {
			throw new NoSuchTimerException();
		}
		Timer timer = notifyTable.get(listener);
		if (timer == null) {
			throw new NoSuchTimerException(listener);
		}
		notifyTable.remove(listener);
		if (notifyTable.size() == 0) {
			active = false;
			timerThread.interrupt();
		}
	}
	
	protected void expired(Timer timer) {
		TimerListener listener = timer.getTimerListener();
		listener.expired(new TimerEvent(listener));
		notifyTable.remove(listener);
		if (notifyTable.size() == 0) {
			active = false;
			timerThread.interrupt();
		}
	}

	public void run() {
		while(active) {
			Enumeration<Timer> en = notifyTable.elements();
			while(en.hasMoreElements()) {
				Timer timer = en.nextElement();
				if(timer.isExpired()) {
					System.out.println("Expired: "+timer);
					expired(timer);
				}     
			}
			try {
				Thread.sleep(period);
			}
			catch(Exception e) {
			}
		}
	}
}
