package agent.wordcount;

//-*- mode:java; encoding:utf8n; coding:utf-8 -*-
//vim:set fileencoding=utf-8:
//http://terai.xrea.jp/Swing/AddRow.html
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;

public class MainPanel extends JPanel {
  private final TestModel model = new TestModel();
  private final JTable table;
  public MainPanel() {
      super(new BorderLayout());
      table = new JTable(model) {
          private final Color evenColor = new Color(250, 250, 250);
          public Component prepareRenderer(TableCellRenderer tcr, int row, int column) {
              Component c = super.prepareRenderer(tcr, row, column);
              if(isRowSelected(row)) {
                  c.setForeground(getSelectionForeground());
                  c.setBackground(getSelectionBackground());
              }else{
                  c.setForeground(getForeground());
                  c.setBackground((row%2==0)?evenColor:getBackground());
              }
              return c;
          }
      };

      TableColumn col = table.getColumnModel().getColumn(0);
      col.setMinWidth(60);
      col.setMaxWidth(60);
      col.setResizable(false);

      model.addTest(new KVEntry("Name 1", "comment..."));
      model.addTest(new KVEntry("Name 2", "Test"));
      model.addTest(new KVEntry("Name d", ""));
      model.addTest(new KVEntry("Name c", "Test cc"));
      model.addTest(new KVEntry("Name b", "Test bb"));
      model.addTest(new KVEntry("Name a", ""));
      model.addTest(new KVEntry("Name 0", "Test aa"));

      table.setAutoCreateRowSorter(true);
      table.setFillsViewportHeight(true);
      table.setComponentPopupMenu(new TablePopupMenu());
      add(new JScrollPane(table));
      setPreferredSize(new Dimension(320, 200));
  }

  class TestCreateAction extends AbstractAction{
      public TestCreateAction(String label, Icon icon) {
          super(label,icon);
      }
      public void actionPerformed(ActionEvent evt) {
          testCreateActionPerformed(evt);
      }
  }
  private void testCreateActionPerformed(ActionEvent e) {
      model.addTest(new KVEntry("New name", ""));
//      Rectangle rect = table.getCellRect(model.getRowCount()-1, 0, true);
//      table.scrollRectToVisible(rect);
  }

  class DeleteAction extends AbstractAction{
      public DeleteAction(String label, Icon icon) {
          super(label,icon);
      }
      public void actionPerformed(ActionEvent evt) {
          deleteActionPerformed(evt);
      }
  }
  public void deleteActionPerformed(ActionEvent evt) {
      int[] selection = table.getSelectedRows();
      if(selection==null || selection.length<=0) return;
      for(int i=selection.length-1;i>=0;i--) {
          model.removeRow(table.convertRowIndexToModel(selection[i]));
      }
  }

  private class TablePopupMenu extends JPopupMenu {
      private final Action deleteAction = new DeleteAction("delete", null);
      public TablePopupMenu() {
          super();
          add(new TestCreateAction("add", null));
          //add(new ClearAction("clearSelection", null));
          addSeparator();
          add(deleteAction);
      }
      public void show(Component c, int x, int y) {
          int[] l = table.getSelectedRows();
          deleteAction.setEnabled(l!=null && l.length>0);
          super.show(c, x, y);
      }
  }

  public static void main(String[] args) {
      EventQueue.invokeLater(new Runnable() {
          public void run() {
              createAndShowGUI();
          }
      });
  }
  public static void createAndShowGUI() {
      try{
          UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
      }catch(Exception e) {
          e.printStackTrace();
      }
      JFrame frame = new JFrame("AddRow");
      frame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
      frame.getContentPane().add(new MainPanel());
      frame.pack();
      frame.setLocationRelativeTo(null);
      frame.setVisible(true);
  }
}


class TestModel extends DefaultTableModel {
  private static final ColumnContext[] columnArray = {
      new ColumnContext("No.",     Integer.class, false),
      new ColumnContext("Name",    String.class,  true),
      new ColumnContext("Comment", String.class,  true)
  };
  private int number = 0;
  public void addTest(KVEntry t) {
      Object[] obj = {number, t.getName(), t.getComment()};
      super.addRow(obj);
      number++;
  }
  public boolean isCellEditable(int row, int col) {
      return columnArray[col].isEditable;
  }
  public Class<?> getColumnClass(int modelIndex) {
      return columnArray[modelIndex].columnClass;
  }
  public int getColumnCount() {
      return columnArray.length;
  }
  public String getColumnName(int modelIndex) {
      return columnArray[modelIndex].columnName;
  }
  private static class ColumnContext {
      public final String  columnName;
      public final Class   columnClass;
      public final boolean isEditable;
      public ColumnContext(String columnName, Class columnClass, boolean isEditable) {
          this.columnName = columnName;
          this.columnClass = columnClass;
          this.isEditable = isEditable;
      }
  }
}
class Test{
  private String name, comment;
  public Test(String name, String comment) {
      this.name = name;
      this.comment = comment;
  }
  public void setName(String str) {
      name = str;
  }
  public void setComment(String str) {
      comment = str;
  }
  public String getName() {
      return name;
  }
  public String getComment() {
      return comment;
  }
}
