/*
 * Copyright (c) 2020 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2020/12/4
 */

package agentspace.discovery;

import java.net.URI;

public class AgentServerInfo {
	protected URI uri = null; 
	protected int matpPort = -1;
	protected int heartbeatPort = -1;
	protected long period = -1;
	protected long lastTime = -1;
	protected long expireTime = -1;

	public AgentServerInfo(URI uri) {
		this.uri = uri;
		this.heartbeatPort = heartbeatPort;
		this.period = period;
	}

	public URI getUri() {
		return uri;
	}

	public int getHeartbeatPort() {
		return heartbeatPort;
	}

	public void setHeartbeatPort(int heartbeatPort) {
		this.heartbeatPort = heartbeatPort;
	}
	
	public long getPeriod() {
		return period;
	}

	public void setPeriod(long period) {
		this.period = period;
	}
	
	public long getExpireTime() {
		return expireTime;
	}

	public void setExpiredTime(long expireTime) {
		this.expireTime = expireTime;
	}

	public void setLastTime(long lastTime) {
		this.lastTime = lastTime;
	}

	public long getLastTime() {
		return lastTime;
	}

	public String getHostName() {
		return uri.getHost();
	}
	
	public int getPort() {
		return uri.getPort();
	}
	
	public boolean isExpired(long time) {
		if (time - lastTime > period) {
			return true;
		}
		else {
			return false;
		}
	}

	public boolean equals(Object obj) {
		if (obj instanceof AgentServerInfo) {
			return uri.equals(((AgentServerInfo)obj).getUri());			
		}
		else {
			return false;
		}
	}

	public String toString() {
		String s = uri.toString();
		if (heartbeatPort > 0) {
			s = ", " + heartbeatPort;
		}
		if (lastTime > 0) {
			s = s + ", last_time:" + lastTime + ", ";
		}
		if (period > 0) {
			s = s + ", duration:" + period + "";
		}
		return s;
	}
	
	public int hashCode() {
		return uri.hashCode();		
	}


}