package agentspace.discovery.gomi;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.net.InetAddress;
import java.net.URI;

import agentspace.InvalidURLException;
import agentspace.registry.AlreadyBoundException;
import agentspace.registry.NotBoundException;

public class RuntimeRegistry {
	protected Map<String, AgentServerInfo> neighbours = null;
			
	public RuntimeRegistry() {
		neighbours = new LinkedHashMap<String, AgentServerInfo>();
	}
	
	public AgentServerInfo add(URI uri, int heartbeatPort, long interval) throws AlreadyBoundException {
		System.out.println("RuntimeRegistry: add("+uri+", "+heartbeatPort+","+interval+")");
		if (neighbours.containsKey(uri.toString())) {
			throw new AlreadyBoundException(uri.toString());
		}
		AgentServerInfo asi = new AgentServerInfo(uri, heartbeatPort, interval);
		synchronized(neighbours) {
			neighbours.put(uri.toString(), asi);
		}
		if (listeners != null) {
			Iterator<RuntimeRegistryListener> itr = listeners.iterator();
			while(itr.hasNext()) {
				RuntimeRegistryEvent evt = new RuntimeRegistryEvent(uri, asi);
				itr.next().added(evt);
			}
		}	
		return asi;
	}
	
	public void remove(AgentServerInfo asi) throws NotBoundException {
		remove(asi.getUri());
	} 
	
	public void remove(URI uri) throws NotBoundException {
		System.out.println("RuntimeRegistry: remove("+uri+")");
		AgentServerInfo asi = neighbours.get(uri.toString());
		if (asi == null) {
			throw new NotBoundException(uri.toString());
		}
		synchronized(neighbours) {
			neighbours.remove(uri.toString());
		}
		if (listeners != null) {
			Iterator<RuntimeRegistryListener> itr = listeners.iterator();
			while(itr.hasNext()) {
				RuntimeRegistryEvent evt = new RuntimeRegistryEvent(uri, asi);
				itr.next().removed(evt);
			}
		}
	}

	public AgentServerInfo lookup(URI uri) {
		return neighbours.get(uri.toString());
	}

	public Set<String> getKeys() {
		return neighbours.keySet();
	}

	public Collection<AgentServerInfo> values() {
		return neighbours.values();
	}

	public Iterator<AgentServerInfo> iterator() {
		return neighbours.values().iterator();
	}

	List<RuntimeRegistryListener> listeners = null;

	public void addRuntimeRegistryListener(RuntimeRegistryListener l) {
		if (listeners == null) {
			listeners = new ArrayList<RuntimeRegistryListener>();
		}
		listeners.add(l);
	}

	public void removeRuntimeRegistryListener(RuntimeRegistryListener l) {
		if (listeners != null) {
			listeners.remove(l);
		}
		if (listeners.size() == 0) {
			listeners = null;
		}
	}
}
