package agentspace.discovery;

import java.net.URI;
import java.util.Iterator;

import agentspace.registry.NotBoundException;

public class RuntimeDiscoveryExpire implements Runnable {
	protected RuntimeRegistry registry;
	protected boolean active = true; 
	protected int expired = -1;
	protected int interval = -1;
	protected Thread expireThread = null;

	public RuntimeDiscoveryExpire(RuntimeRegistry registry, int expired, int interval) {
		this.registry = registry;
		this.expired = expired;
		this.interval = interval;
	}

	public void start() {
		active = true;
		if (expireThread == null) {
			expireThread = new Thread(this);
			expireThread.start();
		}
	}

	public void stop() {
		active = false;
		if (expireThread != null) {
			expireThread.interrupt();
			expireThread = null;
		}
	}

	public void run() {
		while(active) {
			try {
				Iterator<URI> i = registry.getKeys().iterator();
				while (i.hasNext()) {
					URI uri = (URI)i.next();
					AgentServerInfo asi = registry.lookup(uri);
					if (asi != null) {
//						System.out.println("AgentRuntimeDiscovery: System.currentTimeMillis:"+System.currentTimeMillis()+" - asi.getLastTime:"+asi.getLastTime() +" = "+ (System.currentTimeMillis() - asi.getLastTime()) +" >? "+expired);
						if (System.currentTimeMillis() - asi.getLastTime() > expired) {
							registry.remove(uri);
						}
					}
				}
				try {
					Thread.sleep(interval);
				} 
				catch (InterruptedException e) {
					break;
				}
			} 
			catch (NotBoundException e) {
				e.printStackTrace();
			}

		}
	}
}
