/*
 * Copyright (c) 2014 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2014/12/3
 */

package agentspace.monitor.network;

import java.net.UnknownHostException;
import java.util.Date;
import java.util.List;
import java.util.Vector;

import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;

import agentspace.discovery.AgentServerInfo;
import agentspace.runtime.AgentContainer;
import agentspace.runtime.AgentStatus;

public class AgentTableNetworkModel extends DefaultTableModel {
	final String[] names = { "Address", "Port", "Status", "Agents", "Type", "Date" };
	final int[] lengths = {50, 30, 30, 100, 30, 100};
	private List list = null;

	JTable theTable = null;

	public AgentTableNetworkModel() {
		super();
		list = new Vector();
		for (int i = 0 ; i < names.length ; i++ ) {
			super.addColumn(names[i]);
		}
	}
	
	public synchronized void setTable(JTable table) {
		theTable = table;
		for (int i = 0; i < names.length; i++) {
			TableColumn column = table.getColumn(names[i]);
			column.setPreferredWidth(lengths[i]);
		}
	}

	public JTable getTable() {
		return theTable;
	}

	public synchronized int getSelectedIndex() {
		int[] rows = getTable().getSelectedRows();
		if (rows != null && rows.length > 0) {
			return rows[0];
		} else {
			return -1;
		}
	}

	public synchronized String getName(int row) {
		if (0 <= row && row < getRowCount()) {
			return (String) getValueAt(row, 0);
		}
		return null;
	}

	public synchronized void addElement(AgentServerInfo asi) {
		list.add(asi);
		Vector rowlist = new Vector();
		rowlist.addElement(asi.getHostName());
		rowlist.addElement(asi.getPort());
		rowlist.addElement("NG");
		rowlist.addElement("NG");
		rowlist.addElement("NG");
		rowlist.addElement("NG");
		super.addRow(rowlist);
	}

	public synchronized void insertElementAt(AgentServerInfo asi, int row) {
		list.add(asi);
		Vector rowlist = new Vector();
		rowlist.addElement(asi.getHostName());
		rowlist.addElement(asi.getPort());
		rowlist.addElement("NG");
		rowlist.addElement("NG");
		rowlist.addElement("NG");
		rowlist.addElement("NG");
		super.insertRow(row, rowlist);
	}

	public synchronized void removeElement(AgentServerInfo asi) {
		int row = list.indexOf(asi);
		System.out.println("AgentTableNework: removeElement with row="+row+", asi="+asi);
		removeElementAt(row);
	}

	public synchronized void removeElementAt(int row) throws ArrayIndexOutOfBoundsException {
		if (0 <= row && row < getRowCount()) {
			super.removeRow(row);
			list.remove(row);
		}
		else {
			throw new ArrayIndexOutOfBoundsException("Illegal row specified: "+row+" agent list table ");
		}
	}

	public synchronized void changeElement(AgentServerInfo asi) {
		int row = list.indexOf(asi);
		replaceElementAt(asi, row);
	}

	public synchronized void replaceElementAt(AgentServerInfo asi, int row) throws ArrayIndexOutOfBoundsException {
		if (0 <= row && row < getRowCount()) {
			list.set(row, asi);
			super.setValueAt(asi.getPort(), row, 0);
			super.setValueAt("NG", row, 1);
			super.setValueAt("NG", row, 2);
			super.setValueAt("NG", row, 3);
			super.setValueAt("NG", row, 4);
			super.setValueAt("NG", row, 5);
		}
		else {
			throw new ArrayIndexOutOfBoundsException("Illegal row specified: "+row+" agent list table ");
		}
	}
}

