/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/7
 */

package agentspace.network.protocol.rmitp;

import java.io.IOException;
import java.io.Serializable;
import java.net.URL;

import agentspace.Future;
import agentspace.IllegalAgentStateException;
import agentspace.InvalidAgentException;
import agentspace.NoSuchAgentException;
import agentspace.network.protocol.Request;
import agentspace.network.protocol.Response;
import agentspace.network.protocol.ResponseFactory;
import agentspace.network.protocol.rmitp.RmitpConstants;
import agentspace.runtime.AgentRuntimeManager;
import agentspace.runtime.loader.MarshallingUtil;
import agentspace.network.AgentProtocolHandler;

public class SessionHandler implements agentspace.network.AgentProtocolHandler {
	Request request;
	AgentRuntimeManager manager;
	
	public SessionHandler() {
		super();
	}

	public void setRequest(Request request) {
		this.request = request;
	}

	public Response handle() {
		ResponseFactory response = new ResponseFactory();
//		System.out.println("SessionHandler: request = "+request);
		response.setProtocol("RMITP");
		response.setVersion("1.0");
		String method = request.getMethod();
		try {
			if (method.equals("INVOKE")) {
				try {
					byte[] content = request.getContent();
					Future future = manager.invoke(request.getURI(), content);
					response.setContent(MarshallingUtil.marshal((Serializable)future.getReply()));
					response.setCode(RmitpConstants.RMITP_OK);
					response.setComment("OK");
				}
				catch (ClassNotFoundException ex) {
					response.setCode(RmitpConstants.RMITP_CLASS_NOT_FOUND);
					response.setComment("ERROR: Class not found");
				}
				catch (InvalidAgentException ex) {
					response.setCode(RmitpConstants.RMITP_ILLEGAL_AGENT_STATE);
					response.setComment("ERROR: Invalid agent exception");
				}
				catch (IOException ex) {
				}
			}
			else if (method.equals("HELLO")) {
				response.setCode(RmitpConstants.RMITP_OK);
				response.setComment("OK");
			}
			else if (method.equals("PUT")) {
				byte[] content = request.getContent();
				manager.load(content);
				response.setCode(RmitpConstants.RMITP_OK);
				response.setComment("OK");
			}			
			else if (method.equals("GET")) {
//				byte[] content = manager.store(request.getURI());
				response.setCode(RmitpConstants.RMITP_OK);
				response.setComment("OK");
			}
			else if (method.equals("FETCH")) {
				URL uri = new URL(request.getURI());
				try {
				byte[] content = manager.store(uri);
				response.setCode(RmitpConstants.RMITP_OK);
				response.setContent(content);
				response.setComment("OK");
				}
				catch (NoSuchAgentException ex) {
					ex.printStackTrace();
					response.setCode(RmitpConstants.RMITP_NOT_FOUND);
					response.setComment("ERROR: No such agent exception: "+uri); 
				}
				catch (IllegalAgentStateException ex) {
					ex.printStackTrace();
					response.setCode(RmitpConstants.RMITP_ILLEGAL_AGENT_STATE);
					response.setComment("ERROR: Illegal agent state exception");
				}
			}
			else {
				response.setCode(RmitpConstants.RMITP_BAD_REQUEST);
				response.setComment("Unknown method: " + method);
			}
//			System.out.println("SessionHandler: response = "+response);
		}
		catch (Exception ex) {
			ex.printStackTrace();
			response.setCode(RmitpConstants.RMITP_SERVER_ERROR);
			response.setComment("Server errors");
		} 
		return response;
	}

	public void setAgentRuntimeManager(AgentRuntimeManager manager) {
		this.manager = manager;
	}
}