/*
 * Copyright (c) 2014 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

package agentspace.network.protocol.rmitp;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.net.URLConnection;

import agentspace.network.protocol.RequestFactory;
import agentspace.network.protocol.Response;
import agentspace.network.protocol.ResponseParser;

public class URLSessionHandler implements agentspace.network.URLSessionHandler {

	public byte[] handle(String method, URL url, byte[] data) throws IOException {
		byte[] result = null;
		if (url == null) {
			throw new IOException("Illegal URL = "+url);
		}
		try {
			URLConnection connection = url.openConnection();
			OutputStream out = connection.getOutputStream();
			InputStream in = connection.getInputStream();
			RequestFactory request = new RequestFactory();
			request.setProtocol("RMITP");
			request.setVersion("1.0");
			if (method.equals("INVOKE")) {
				request.setMethod("INVOKE");
				request.setURI(url.getFile());
				request.setContent(data);
				out.write(request.toByteArray());
				out.flush();
//				System.out.println("matp.URLSessionHandler: sending "+data.length+" to "+url);
				Response response = new ResponseParser(in);
//				System.out.println("matp.URLSesssionHandler: receiving "+response+" from "+url);
				if (response.getCode() < 200 || response.getCode() >= 300) {
					throw new IOException("Receiving error code "+response.getCode()+
							" ("+response.getComment()+") from server");
				}
			}
		}
		catch (IOException ex) {
			ex.printStackTrace();
		}
		return result;
	}
}
