package agentspace.packet.multicast;

import java.io.IOException;
import java.io.InterruptedIOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.MulticastSocket;
import java.net.NetworkInterface;
import java.net.SocketAddress;
import java.net.URI;
import java.net.URISyntaxException;
import java.nio.charset.StandardCharsets;

import agentspace.packet.Packet;
import agentspace.packet.SizeLimitExceededPacketException;


public class MulticastUDPSender {

	static String GROUPNAME = "228.5.6.7";
	static int PORT = 6789; 

	
	InetAddress mcastaddr;
	InetSocketAddress group;
	NetworkInterface netIf;
	MulticastSocket msocket; 
	protected String groupName;
	protected int port;

	public MulticastUDPSender(String groupName, int port) {
		this.groupName = groupName;
		this.port = port;
		try {
			mcastaddr = InetAddress.getByName(groupName);
			group = new InetSocketAddress(mcastaddr, port);
			netIf = NetworkInterface.getByName("bge0");
			msocket = new MulticastSocket(port);
			msocket.setReuseAddress(true);
			msocket.joinGroup(group, netIf);
		} 
		catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void close() throws IOException {
		msocket.leaveGroup(group, netIf);
		msocket.close();
	}

	public void send(byte[] data) throws IOException {
		DatagramPacket hi = new DatagramPacket(data, data.length, group);
		msocket.send(hi);
//		System.out.println("SenderHandler: send("+data.length+")");
	}
	/*	
	public static void main(String[] args) {
		boolean flag = true;
		for (int i = 0; i < args.length; i++) {
			if ((args[i].equals("-onlysender") || args[i].equals("-s"))	&& (i + 1 < args.length)) {
				try {
					flag = false;
				} 
				catch (NumberFormatException ex) {
					System.exit(1);
				}
			}
		}
		try {
			MulticastUDPSender sender = new MulticastUDPSender(GROUPNAME, PORT, flag);
		} 
		catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public MulticastUDPSender(String groupName, int port, boolean flag) {
		Thread t = new Thread(new ReceiverHandler(groupName, port));
		t.start();
		SenderHandler sender;
		if (flag) {
			sender = new SenderHandler(groupName, port);
			for (int i = 0 ; i < 10 ; i++ ) {
				try {Thread.sleep(100);} catch (InterruptedException e) {e.printStackTrace();}
				String msg = "Hello";
				byte[] data = msg.getBytes(StandardCharsets.UTF_8);
				sender.send(data, groupName, port);
			}
		}
		try {
			Thread.sleep(100);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	};

	class SenderHandler {
		InetAddress mcastaddr;
		InetSocketAddress group;
		NetworkInterface netIf;
		MulticastSocket msocket; 
		protected String groupName;
		protected int port;
		public SenderHandler(String groupName, int port) {
			this.groupName = groupName;
			this.port = port;
			try {
				mcastaddr = InetAddress.getByName(groupName);
				group = new InetSocketAddress(mcastaddr, port);
				netIf = NetworkInterface.getByName("bge0");
				msocket = new MulticastSocket(port);
				msocket.joinGroup(group, netIf);
			} 
			catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
		public void send(byte[] data, String groupName, int port) {
			try {
				InetAddress mcastaddr = InetAddress.getByName(groupName);
				InetSocketAddress group = new InetSocketAddress(mcastaddr, port);
				NetworkInterface netIf = NetworkInterface.getByName("bge0");
				MulticastSocket msocket = new MulticastSocket(port);
				msocket.joinGroup(group, netIf);
				DatagramPacket hi = new DatagramPacket(data, data.length, group);
				msocket.send(hi);
				System.out.println("SenderHandler: send("+data.length+")");
				//				send((new String("Hello")).getBytes(), port);
				msocket.leaveGroup(group, netIf);
				msocket.close();
			}
			catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	class ReceiverHandler implements Runnable {
		MulticastUDPReceiver receiver;
		protected MulticastSocket msocket = null; 
		protected DatagramSocket socket = null;
		protected InetAddress group = null;
		protected String groupName;
		protected int port;

		public ReceiverHandler(String groupName, int port) {
			this.groupName = groupName;
			this.port = port;
			try {
				receiver = new MulticastUDPReceiver(groupName, port);
		
			} 
			catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		public void run() {
			System.out.println("ReceiverHandler: starting..");
			while (true) {
				try {
					byte[] buf = receiver.receive();
					datagram.getData();
					System.out.println("ReceiverHandler: receive("+buf.length+") in size="+buf.length);
				}
				catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}		
			}				
		}	
	}
	*/
}