package agentspace.packet;

import java.io.IOException;
import java.net.UnknownHostException;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import agentspace.packet.Packet;
import agentspace.packet.PacketListener;
import agentspace.packet.SizeLimitExceededPacketException;
import agentspace.packet.multicast.MulticastUDPSender;

public class HeartbeatManager implements Runnable {
	protected boolean active = false;
	protected MulticastUDPSender sender;
	protected long interval;
	protected Packet packet;
	protected Thread heatbeatThread;

	public HeartbeatManager(MulticastUDPSender sender, Packet packet, int interval) {
		this(sender, interval);
		this.packet = packet;
	}

	public HeartbeatManager(MulticastUDPSender sender, int interval) {
		this.sender = sender;
		this.interval = interval;
	}
	
	public long getInterval() {
		return interval;
	}

	public void setInterval(long interval) {
		this.interval = interval;
	}

	public void setPacket(Packet packet) {
		this.packet = packet;
	}

	public Packet getPacket(Packet packet) {
		return packet;
	}

	public void start() {
		active = true;
		if (heatbeatThread == null) {
			heatbeatThread = new Thread(this);
			heatbeatThread.start();
		}
	}

	public void stop() {
		active = false;
		if (heatbeatThread != null) {
			heatbeatThread.interrupt();
			heatbeatThread = null;
		}
	}

	public void run() {
		while(active) {
			try {
//				System.out.println("AgentRuntimeHeatbeat: sending");
				sender.send(packet.getBytes());
				try {
					Thread.sleep(interval);
				} 
				catch (InterruptedException e) {
					break;
				}
			} 
			catch (IllegalArgumentException e) {
				e.printStackTrace();
			} 
			catch (SizeLimitExceededPacketException e) {
				e.printStackTrace();
			}
			catch (UnknownHostException e) {
				e.printStackTrace();
			}
			catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
}
