/*
 * Copyright (c) 2012 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/11, Modified on 2012/12/20
 */

package agentspace.registry;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.List;

import agentspace.AgentIdentifier;
import agentspace.NoSuchAgentException;
import agentspace.runtime.AgentContainer;

public class AgentRegistry {
	protected Hashtable<AgentIdentifier, AgentContainer> table = null;

	public AgentRegistry() {
		table = new Hashtable<AgentIdentifier, AgentContainer>();
	}

	public void add(AgentContainer ac) {
//		System.out.println("AgentRegistry: add(AgentContainer"+ac+")");
		table.put(ac.getIdentifier(), ac);
	}
	
	public void remove(AgentContainer ac) throws NotBoundException {
//		System.out.println("AgentRegistry: remove(AgentContainer"+ac+")");
		if (table.remove(ac.getIdentifier()) == null) {
			throw new NotBoundException();
		}
	}
	
	public AgentContainer[] lookup(String name) throws NotBoundException {
		List<AgentContainer> list = new ArrayList<AgentContainer>();
		Iterator<AgentContainer> it = table.values().iterator();
		while (it.hasNext()) {
			AgentContainer ac = it.next(); 
			if (name.equals(ac.getName())) {
				list.add(ac);
			}
		}
		if (list.isEmpty()) {
			throw new NotBoundException("No such element named"+name);
		}
		AgentContainer[] acs = new AgentContainer[list.size()];
		for (int i = 0 ; i < list.size() ; i++ ) {
			acs[i] = list.get(i);		
		}
		return acs;
	}
	
	public AgentContainer getContainer(AgentIdentifier aid) throws NoSuchAgentException {
		AgentContainer ac = table.get(aid);
		Object obj = table.get(aid);
		if (obj != null) {
			return ac;
		}
		throw new NoSuchAgentException(aid.toString());
	}
	
	public int size() {
		return table.size();
	}
	
	public AgentContainer[] getContainers() {
		AgentContainer[] acs = new AgentContainer[size()];
		Enumeration<AgentContainer> e = table.elements();
		for (int i = 0 ; e.hasMoreElements() && i < size() ; i	++ ) {
			acs[i] = e.nextElement(); 
		}
		if (size() == 0) {
			return null;
		}
		else {
			return acs;
		}
	}

	public AgentIdentifier getIdentifier(AgentContainer ac) throws NotBoundException {
		for (AgentContainer a : table.values()) {
			if (a == ac) {
				return a.getIdentifier();
			}
		}
		throw new NotBoundException();
	}		
} 
