/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/5
 */

package agentspace.runtime.event;

import java.io.Serializable;
import java.util.EventObject;

import agentspace.Future;
import agentspace.Message;
import agentspace.runtime.AgentLifecycleManager;

public class AgentLifecycleEvent extends EventObject implements Serializable, Cloneable {
	final public static int CREATE               = 0x01;
	final public static int DESTROY              = 0x02;
	final public static int SUSPEND              = 0x03;
	final public static int RESUME               = 0x04;
	final public static int LEAVE                = 0x05;
	final public static int ARRIVE               = 0x06;
	final public static int DUPLICATE            = 0x07;
	final public static int PARENT               = 0x08;
	final public static int ADD                  = 0x09;
	final public static int REMOVE               = 0x0A;
	final public static int MESSAGE              = 0x0C;
	final public static int HALT                 = 0x10;

	protected int action;
	protected Object result;
	protected AgentLifecycleManager target;
	protected Message message;
	protected Future future;
	protected boolean finished = false;
	protected Exception exception = null;
	
	public AgentLifecycleEvent(Object source, AgentLifecycleManager target) {
		super((Object) source);
		this.target = target;
	}

	public AgentLifecycleEvent(Object source, AgentLifecycleManager target, int action) {
		this((Object) source, target);
		this.action = action;
	}

	public Object getSource() {
		return source;
	}

	public AgentLifecycleManager getContainer() {
		return target;
	}

	public int getAction() {
		return action;
	}

	public void setAction(int action) {
		this.action = action;
	}

	public Message getMessage() {
		return message;
	}
	
	public void setMessage(Message message) {
		this.message = message;
	}

	public String getActionName() {
		return getActionName(action); 
	}
	
	public synchronized void done() {
		while (!this.finished) {
			try{
				wait();
			}
			catch (InterruptedException e) {}
		}
	}
	
	public synchronized void setFinished(boolean finished) {
//  		System.out.println("AgentLifecycleEvent: setFinished("+finished+")");
		this.finished =finished; 
		notify();
	}
	
	public Object getResult() {
		return result;
	}

	public void setResult(Object result) {
		this.result = result;
	}

	public static String getActionName(int act) {
		switch(act) {
		case CREATE:
			return "create";
		case DESTROY:
			return "destroy";
		case SUSPEND:
			return "suspend";
		case RESUME:
			return "resume";
		case LEAVE:
			return "leave";
		case ARRIVE:
			return "arrive";
		case PARENT:
			return "parent";
		case DUPLICATE:
			return "duplicate";
		case ADD:
			return "add";
		case REMOVE:
			return "remove";
		case MESSAGE:
			return "message";
		case HALT:
			return "halt";
		default:
			return "unknown";
		}
	}

	public Exception getException() {
		return exception;
	}

	public void setException(Exception exception) {
		this.exception = exception;
	}

	/**
	 * @return Returns the future.
	 */
	public Future getFuture() {
		return future;
	}
	/**
	 * @param future The future to set.
	 */
	public void setFuture(Future future) {
		this.future = future;
	}
	
	public String toString() {
		return "AgentLifecycleEvent: src=" + source + ", trgt=" + target + 
		", action=" + getActionName(action);
	}
}