package agentspace.shell;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Properties;

import agentspace.AgentIdentifier;
import agentspace.IllegalAgentStateException;
import agentspace.InvalidURLException;
import agentspace.NoSuchAgentException;
import agentspace.context.ContextManager;
import agentspace.runtime.AgentContainer;
import agentspace.tools.FileUtil;

public class AgentCommandHandler extends CommandHandler {
	protected ContextManager context;

	public AgentCommandHandler(ContextManager context,	String line, Properties environ) {
		super(line, environ);
		this.context = context;
	}

	public AgentCommandHandler(ContextManager context, String line) {
		this(context, line, null);
	}

	public void setContext(ContextManager context) {
		this.context = context;
	}

	protected String eval_command(String command, String[] args) {
		try {
			if (command.equalsIgnoreCase("load")) {
				return eval_load(command, args);
			}
			else if (command.equalsIgnoreCase("exit")) {
				return eval_exit(command, args);
			}
			else if (command.equalsIgnoreCase("quit")) {
				return eval_exit(command, args);
			}
			else if (command.equalsIgnoreCase("help")) {
				return eval_help(command, args);
			}
			else if (command.equalsIgnoreCase("gc")) {
				return eval_gc(command, args);
			}
			else if (command.equalsIgnoreCase("env")) {
				return eval_env(command, args);
			}
			else if (command.equalsIgnoreCase("set")) {
				return eval_set(command, args);
			}
			else if (command.equalsIgnoreCase("list")) {
				return eval_list(command, args);
			}
			else if (command.equalsIgnoreCase("sleep")) {
				return eval_sleep(command, args);
			}
		}
		catch (IllegalArgumentException e) {
			return "ERROR: "+e;
		}
		return "ERROR: UNKNOWN COMMAND: "+command;
	}

	protected String eval_load(String command, String[] args) throws IllegalArgumentException {
		try {
			if (args.length == 1) {
				byte[] data = FileUtil.fileToByteArray(args[0]);
				return "OK: "+context.load(data);
			}
			else {
				throw new IllegalArgumentException("create <filename>");
			}
		}
		catch (Exception e) {
			e.printStackTrace();
			throw new IllegalArgumentException("ERROR: "+e.getMessage());
		}
	}
	
	protected String eval_move(String command, String[] args) throws IllegalArgumentException {
		if (args.length != 3) {
			throw new IllegalArgumentException("move <agent-identifier> <url>");
		}
		AgentIdentifier id = new AgentIdentifier(args[1]);
		URL url;
		try {
			url = new URL(args[2]);
		} 
		catch (MalformedURLException e) {
			e.printStackTrace();
			return "ERROR: Invalid URL";
		}			
		AgentContainer ac;
		try {
			ac = context.getContainer(id);
		} 
		catch (NoSuchAgentException e) {
			e.printStackTrace();
			return "ERROR: Invalid agent identifier";
		}
		try {
			context.move(ac, url, true, false);
		} 
		catch (IOException e) {
			e.printStackTrace();
			return "ERROR: Network problem";
		} 
		catch (InvalidURLException e) {
			e.printStackTrace();
			return "ERROR: Invalid URL";
		} 
		catch (IllegalAgentStateException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "ERROR: Illegal Agent State";
		}
		return "MOVE";
	}
	
	

	
	protected String eval_sleep(String command, String[] args) {
		if (args.length != 1) {
			throw new IllegalArgumentException("sleep <number>");
		}
		try {
			int interval = Integer.parseInt(args[0]);
			Thread.sleep(interval);
			return "OK: "+interval;
		}
		catch (NumberFormatException e){
			return "ERROR: Number Format Exception: "+args[0];
		}
		catch (Exception e) {
			return "ERROR: unknown";
		}
	}

	protected String eval_set(String command, String[] args) {
		if (args.length != 2) {
			throw new IllegalArgumentException("set <param-name> <value>");
		}
		environ.put(args[0], args[1]);
		return "OK: "+args[0]+" = "+args[1];
	}

	protected String eval_env(String command, String[] args) {
		if (args.length > 1) {
			throw new IllegalArgumentException("set <param-name> <value>");
		}
		String result = "";
		for (Object key : environ.keySet()) {
			result = result = key + "\t" + environ.get(key) + "\n";
		}
		return result;
	}

	protected String eval_exit(String command, String[] args) {
		if (args.length != 0) {
			throw new IllegalArgumentException("exit");
		}
		System.out.println("the system finishing when running at AgentCommandHandler");
		System.exit(0);
		return "EXIT";
	}

	protected String eval_gc(String command, String[] args) {
		if (args.length != 1) {
			throw new IllegalArgumentException("gc");
		}
		System.gc();
		return "GC";
	}

	protected String eval_help(String command, String[] args) {
		String line = "";
		line = line + "list" + "\n";
		line = line + "load <filename>" + "\n";
		line = line + "sleep <num>" + "\n";
		line = line + "env [<var>]" + "\n";
		line = line + "set <var> <value>" + "\n";
		line = line + "gc" + "\n";
		line = line + "exit" + "\n";
		return line;
	}

	protected String eval_list(String command, String[] args) throws IllegalArgumentException {
		if (args.length != 0) {
			throw new IllegalArgumentException("list");
		}
		String result = "";
		if (args.length == 0) {
			AgentContainer[] containers = context.getContainers();
			if (containers == null) {
				result = "None agent";
			}
			else {
				for (int i = 0 ; i < containers.length ; i++ ) {
					result = result + containers[i].getIdentifier() + " "+ 
					containers[i].getName()+" "+
					containers[i].getType() +" "+
					containers[i].getStatus() + "\n";
				}
			}
		}
		return result;
	}

	public String toString() {
		try {
			return getResult().toString();
		}
		catch (Exception e) {
			return e.getMessage();
		}
	}
}

